
import subprocess
import os
from typing import Union, Tuple


class SSH:
    user: str
    port: int
    host: str
    ssh_opts: str
    ssh_bin: str
    timeout: int
    sshpass_bin: str
    known_hosts_file: str

    def __init__(self, user: str, port: int, host: str, ssh_opts: str, private_key: str,
                 ssh_bin: str, password: str, timeout: int, sshpass_bin: str, known_hosts_file: str):
        self.user = user
        self.port = port
        self.host = host
        self.ssh_opts = ssh_opts
        self.ssh_bin = ssh_bin
        self.password = password
        self.timeout = timeout
        self.sshpass_bin = sshpass_bin
        self.known_hosts_file = os.path.expanduser(known_hosts_file if known_hosts_file else '~/.ssh/known_hosts')

        if self.known_hosts_file:
            self.ssh_opts += ' -o UserKnownHostsFile=' + self.known_hosts_file

        if private_key:
            self.ssh_opts += ' -i %s ' % private_key

    def execute(self, remote_command: Union[str, list]) -> Tuple[str, int]:
        self._make_sure_host_is_known()

        local_command = [self.ssh_bin]

        if self.password:
            local_command = [self.sshpass_bin, '-p', self.password] + local_command

        if self.ssh_opts:
            local_command += self.ssh_opts.strip().split(' ')

        local_command += [
            '-p', str(self.port),
            self.user + '@' + self.host
        ]

        if type(remote_command) == str:
            local_command.append(remote_command)
        else:
            local_command += remote_command

        try:
            out = subprocess.check_output(local_command, stderr=subprocess.STDOUT, timeout=self.timeout).decode('utf-8')
            exit_code = 0
        except subprocess.CalledProcessError as e:
            out = e.output.decode('utf-8')
            exit_code = e.returncode

        return out, exit_code

    def _make_sure_host_is_known(self):
        pattern = self.host if self.port == 22 else '[' + self.host + ']:' + str(self.port)
        os.system('mkdir -p $(dirname %s); touch %s' % (self.known_hosts_file, self.known_hosts_file))

        with open(self.known_hosts_file, 'rb') as f:
            current_content = f.read().decode('utf-8')

            if pattern not in current_content:
                self._add_host_to_known_hosts()

    def _add_host_to_known_hosts(self):
        subprocess.call(
            'ssh-keyscan -p %i %s >> %s' % (self.port, self.host, self.known_hosts_file), shell=True,
            stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
        )



import unittest
import docker

from .utils import run_check
from .utils.ssh_test import TestThatRequiresSshServer


class SshCommandCheckTest(TestThatRequiresSshServer, unittest.TestCase):
    docker_client: docker.DockerClient

    def test_not_passed_host_raises_human_readable_message(self):
        stdout: str
        result: int

        stdout, result, hooks_output = run_check('ssh-files-checksum', {}, {})

        self.assertIn('HOST is mandatory', stdout)
        self.assertFalse(result)

    def test_success_case(self):
        stdout: str
        result: int

        stdout, result, hooks_output = run_check('ssh-files-checksum', {
            'HOST': 'localhost',
            'PORT': 3222,
            'USER': 'root',
            'PASSWORD': 'root',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
            'EXPECTS': {
                "/dev/null": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
            }
        }, {})

        self.assertEqual('All checksums are matching', stdout.strip())
        self.assertTrue(result)

    def test_at_least_one_checksum_not_matching(self):
        stdout: str
        result: int

        stdout, result, hooks_output = run_check('ssh-files-checksum', {
            'HOST': 'localhost',
            'PORT': 3222,
            'USER': 'root',
            'PASSWORD': 'root',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
            'EXPECTS': {
                "/dev/null": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
                "/bin/sh": "will-not-match-this-one"
            }
        }, {})

        self.assertIn("FAIL: '/bin/sh' checksum is not matching. Expected: 'will-not-match-this-one'", stdout.strip())
        self.assertFalse(result)




import docker
import docker.errors
import subprocess
import time


class TestThatRequiresSshServer:
    docker_client: docker.DockerClient

    @classmethod
    def setUpClass(cls) -> None:
        TestThatRequiresSshServer.docker_client = docker.from_env()
        TestThatRequiresSshServer._remove_ssh_container()
        TestThatRequiresSshServer.docker_client.containers.run('sickp/alpine-sshd:7.5', name='ssh', ports={'22/tcp': 3222}, detach=True)
        TestThatRequiresSshServer._wait_for_ssh_to_be_ready()

    @classmethod
    def tearDownClass(cls) -> None:
        TestThatRequiresSshServer._remove_ssh_container()
        TestThatRequiresSshServer.docker_client.close()

    @staticmethod
    def _remove_ssh_container():
        try:
            container = TestThatRequiresSshServer.docker_client.containers.get('ssh')

            try:
                container.kill()
            except docker.errors.APIError:
                pass

            container.remove()

        except docker.errors.NotFound:
            pass

    @staticmethod
    def get_current_ssh_server_fingerprint():
        return subprocess.check_output(
            'ssh-keyscan -t rsa -p 3222 localhost', stderr=subprocess.DEVNULL, shell=True
        ).decode('utf-8')

    @staticmethod
    def _wait_for_ssh_to_be_ready():
        out = ''

        for i in range(1, 600):
            try:
                out += str(subprocess.check_output('echo "ttttt\n\n" | nc -w 1 "localhost" "3222"',
                                                   shell=True, stderr=subprocess.STDOUT))

                if "SSH-2.0-OpenSSH" in out:
                    return
            except subprocess.CalledProcessError:
                time.sleep(0.5)

        raise Exception('SSH container did not get up properly. Output: ' + out)




import inspect
import sys
import os

path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../'
sys.path.insert(0, path)

from infracheck.infracheck.runner import Runner


def run_check(check_type: str, input_data: dict, hooks: dict):
    runner = Runner([path + '/../../example/healthchecks', path + '/../infracheck/'])
    return runner.run(check_type, input_data, hooks)




import os
import posix
import sqlite3
import json
import time


class Repository:
    checks_dirs: list
    configured_dirs: list
    db: sqlite3.Connection

    def __init__(self, project_dirs: list, db_path: str = '~/.infracheck.sqlite3'):
        self.checks_dirs = []
        self.configured_dirs = []
        self._connect_to_db(db_path)

        for path in project_dirs:
            self.checks_dirs.append(path + '/checks')
            self.configured_dirs.append(path + '/configured')

    def _connect_to_db(self, db_path: str):
        self.db = sqlite3.connect(os.path.expanduser(db_path))

        try:
            self.db.execute(
                '''
                    CREATE TABLE checks_cache (
                        id INTEGER PRIMARY KEY AUTOINCREMENT, 
                        check_name TEXT, 
                        data TEXT, 
                        date_added TEXT
                    );
                '''
            )
        except sqlite3.OperationalError:
            pass

        try:
            self.db.execute('CREATE INDEX check_name_index ON checks_cache(check_name);')
        except sqlite3.OperationalError:
            pass

        self.db.commit()

    def get_configured_checks(self, with_disabled=False):
        found = []

        for path in self.configured_dirs:
            if not os.path.isdir(path):
                continue

            configurations = os.scandir(path)

            for entry in configurations:
                if not with_disabled and not entry.name.endswith('.json'):
                    continue
                elif with_disabled and not (entry.name.endswith('.disabled') or entry.name.endswith('.json')):
                    continue

                found.append(os.path.splitext(entry.name)[0].replace('.json', ''))

        return list(set(found))

    def get_available_checks(self):
        results = []

        for path in self.checks_dirs:
            available_checks = os.scandir(path)

            def map_checks_to_name(check: posix.DirEntry):
                return str(check.name)

            results += map(map_checks_to_name, available_checks)

        return list(set(results))

    def _purge_cache(self, check_name: str):
        self.db.execute('DELETE FROM checks_cache WHERE check_name = ?', [check_name])

    def push_to_cache(self, check_name: str, data):
        self._purge_cache(check_name)
        self.db.execute(
            '''
                INSERT INTO checks_cache (id, check_name, data, date_added)
                VALUES (NULL, ?, ?, ?);
            ''',
            [check_name, json.dumps(data), time.time()]
        )

        self.db.commit()

    def retrieve_cache(self, check_name: str):
        cursor = self.db.execute('SELECT data FROM checks_cache WHERE check_name = ?', [check_name])
        data = cursor.fetchone()

        if not data:
            return None

        try:
            return json.loads(data[0])
        except:
            return None



import unittest
import os
import inspect
from typing import Tuple

# import a script with "-" in filename
path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../'
with open(path + '/infracheck/checks/docker-container-log', 'r') as script:
    exec(script.read())


class DockerContainerLogCheckTest(unittest.TestCase):
    @staticmethod
    def _run_mocked_check(container: str, regexp: str, max_lines: int, since: int, should_be_present: bool) \
            -> Tuple[bool, str, list]:

        check = DockerContainerLogCheck()
        called = []

        def check_output(*args, **kwargs):
            called.append(args)

            return b"""
            Hello
            World
            
            This
            Is
            A
            Test
            """

        check._check_output = check_output
        result, message = check.main(
            container,
            regexp,
            max_lines,
            since,
            should_be_present
        )

        return result, message, called

    def test_expecting_exact_match_and_have_match(self) -> None:
        result, message, called = self._run_mocked_check(
            container='container_1',
            regexp='Hello',
            max_lines=15,
            since=5,
            should_be_present=True
        )

        self.assertTrue(result)
        self.assertEqual('The container last 15 lines of output has a match, as expected', message)

    def test_expecting_that_will_not_match(self) -> None:
        result, message, called = self._run_mocked_check(
            container='container_1',
            regexp='This text will not be found',
            max_lines=15,
            since=5,
            should_be_present=True
        )

        self.assertFalse(result)
        self.assertEqual('The container last 15 lines of output are not matching, expecting they were', message)

    def test_expecting_to_not_find_text_and_will_not_find(self) -> None:
        result, message, called = self._run_mocked_check(
            container='container_1',
            regexp='This text will not be found',
            max_lines=15,
            since=5,
            should_be_present=False
        )

        self.assertTrue(result)
        self.assertEqual('The container last 15 lines of output are not matching, as expected', message)

    def test_expecting_to_not_find_but_will_find(self) -> None:
        result, message, called = self._run_mocked_check(
            container='container_1',
            regexp='Hello',
            max_lines=15,
            since=5,
            should_be_present=False
        )

        self.assertFalse(result)
        self.assertEqual('The container output is matching the pattern but should not, '
                         'looked at 15 lines since 5s in container_1', message)



#!/usr/bin/env python3

import os
from json import loads as json_loads

assert os.path.isfile('Pipfile.lock')

requirements = ""

with open('./Pipfile.lock', 'rb') as f:
    piplock = f.read()

for package_name, details in json_loads(piplock)['default'].items():
    requirements += package_name + details['version'] + "\n"

with open('./requirements.txt', 'wb') as f:
    f.write(requirements.encode('utf-8'))



import unittest
import docker

from .utils import run_check
from .utils.ssh_test import TestThatRequiresSshServer


class SshFingerprintTest(TestThatRequiresSshServer, unittest.TestCase):
    docker_client: docker.DockerClient

    def test_success_case(self):
        stdout: str
        result: int

        # we create SSH server for testing dynamically in a docker container, so each time it has a different identity
        current_expected_fingerprint = self.get_current_ssh_server_fingerprint()

        stdout, result, hooks_output = run_check('ssh-fingerprint', {
            'HOST': 'localhost',
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': current_expected_fingerprint
        }, {})

        self.assertEqual('Fingerprint is OK', stdout.strip())
        self.assertTrue(result)

    def test_invalid_fingerprint(self):
        stdout: str
        result: int

        stdout, result, hooks_output = run_check('ssh-fingerprint', {
            'HOST': 'localhost',
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': 'SOME FINGERPRINT THAT DOES NOT MATCH SERVER FINGERPRINT'
        }, {})

        self.assertIn('Fingerprint does not match', stdout.strip())
        self.assertFalse(result)

    def test_missing_host_parameter(self):
        stdout: str
        result: int

        stdout, result, hooks_output = run_check('ssh-fingerprint', {
            'PORT': 3222,
            'EXPECTED_FINGERPRINT': 'SOME FINGERPRINT THAT DOES NOT MATCH SERVER FINGERPRINT'
        }, {})

        self.assertIn('You need to provide a HOST', stdout.strip())
        self.assertFalse(result)
